<meta http–equiv=“Content-Type” content=“text/html; charset=utf-8” />
<?php

require 'ekler.php';

$tr = new Turkish;


$iller = array('Adana', 'Adıyaman', 'Afyon', 'Ağrı', 'Amasya', 'Ankara', 'Antalya', 'Artvin',
'Aydın', 'Balıkesir', 'Bilecik', 'Bingöl', 'Bitlis', 'Bolu', 'Burdur', 'Bursa', 'Çanakkale',
'Çankırı', 'Çorum', 'Denizli', 'Diyarbakır', 'Edirne', 'Elazığ', 'Erzincan', 'Erzurum', 'Eskişehir',
'Gaziantep', 'Giresun', 'Gümüşhane', 'Hakkari', 'Hatay', 'Isparta', 'Mersin', 'İstanbul', 'İzmir', 
'Kars', 'Kastamonu', 'Kayseri', 'Kırklareli', 'Kırşehir', 'Kocaeli', 'Konya', 'Kütahya', 'Malatya', 
'Manisa', 'Kahramanmaraş', 'Mardin', 'Muğla', 'Muş', 'Nevşehir', 'Niğde', 'Ordu', 'Rize', 'Sakarya',
'Samsun', 'Siirt', 'Sinop', 'Sivas', 'Tekirdağ', 'Tokat', 'Trabzon', 'Tunceli', 'Şanlıurfa', 'Uşak',
'Van', 'Yozgat', 'Zonguldak', 'Aksaray', 'Bayburt', 'Karaman', 'Kırıkkale', 'Batman', 'Şırnak',
'Bartın', 'Ardahan', 'Iğdır', 'Yalova', 'Karabük', 'Kilis', 'Osmaniye', 'Düzce');


foreach ($iller as $veri) {
echo $tr->makeGenitive($veri, array ( "proper_noun" => true ) ), PHP_EOL;
echo "<br>";
	echo $tr->makeDative($veri, array ( "proper_noun" => true ) ), PHP_EOL;
echo "<br>";
	echo $tr->makeAblative($veri, array ( "proper_noun" => true ) ), PHP_EOL;
	echo "<br>";
	echo $tr->makeAccusative($veri, array ( "proper_noun" => true ) ), PHP_EOL;
	echo "<br>";

	echo $tr->makeDerivative($veri, array ( "proper_noun" => false ) ), PHP_EOL;
	echo "<br>";

	echo $tr->makeGenitive($veri, array ( "proper_noun" => false ) ), PHP_EOL;
	echo "<br>";
	echo $tr->makeAccusative($veri, array ( "proper_noun" => false ) ), PHP_EOL;
	echo "<br>";
	echo "<br>";

}




// echo $tr->makeDative("Fatma", array ( "proper_noun" => true ) );